@@ -1,26 +1,28 @@
-#' Calculates Multinomial Logit probabilities
+#' Calculates class allocation probabilities for a Latent Class model
 #'
-#' Calculates probabilities of a Multinomial Logit model.
+#' Calculates class allocation probabilities for a Latent Class model using a Multinomial Logit model and can also perform other operations based on the value of the \code{functionality} argument.
 #'
 #' @param classAlloc_settings List of inputs of the MNL model. It should contain the following.
 #'                     \itemize{
-#'                       \item \strong{\code{V}}: Named list of deterministic utilities . Utilities of the classes in class allocation model. Names of elements must match those in \code{avail}, if provided.
-#'                       \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of classes in class allocation model, one element per class Names of elements must match those in \code{V}. Values can be 0 or 1.
-#'                       \item \strong{\code{rows}}: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                       \item \strong{\code{utilities}}: Named list of deterministic utilities . Utilities of the classes in class allocation model. Names of elements must match those in \code{avail}, if provided.
+#'                       \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of classes, one element per class Names of elements must match those in \code{classes}. Values can be 0 or 1. These can be scalars or vectors (of length equal to rows in the database). A user can also specify \code{avail=1} to indicate universal availability, or omit the setting completely.
+#'                       \item \strong{\code{rows}}: Boolean vector. Consideration of which rows to include. Length equal to the number of observations (nObs), with entries equal to TRUE for rows to include, and FALSE for rows to exclude. Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
 #'                       \item \strong{\code{componentName}}: Character. Name given to model component.
 #'                     }
-#' @return The returned object depends on the value of argument \code{functionality}, which it fetches from the calling stack.
+#' @return The returned object depends on the value of argument \code{functionality}, which it fetches from the calling stack (see \link{apollo_validateInputs}).
 #'         \itemize{
-#'         "estimate","conditionals", "components", "output", "prediction", "raw", "report"
-#'           \item \strong{\code{"estimate"}}: vector/matrix/array. Returns the probabilities for all classes
-#'           \item \strong{\code{"prediction"}}: Same as \code{"estimate"}.
-#'           \item \strong{\code{"validate"}}: Same as \code{"estimate"}, but it also runs a set of tests to validate the function inputs.
-#'           \item \strong{\code{"zero_LL"}}: Returns a scalar NA.
-#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}.
 #'           \item \strong{\code{"components"}}: Same as \code{"estimate"}.
+#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}.
+#'           \item \strong{\code{"estimate"}}: List of vector/matrices/arrays with the allocation probabilities for each class.
+#'           \item \strong{\code{"gradient"}}: List containing the likelihood and gradient of the model component.
 #'           \item \strong{\code{"output"}}: Same as \code{"estimate"}.
+#'           \item \strong{\code{"prediction"}}: Same as \code{"estimate"}.
+#'           \item \strong{\code{"preprocess"}}: Returns a list with pre-processed inputs, based on \code{classAlloc_settings}.
 #'           \item \strong{\code{"raw"}}: Same as \code{"estimate"}.
-#'           \item \strong{\code{"repor"}}: Same as \code{"estimate"}.
+#'           \item \strong{\code{"report"}}: Same as \code{"estimate"}.
+#'           \item \strong{\code{"shares_LL"}}: List with probabilities for each class in an equal shares setting.
+#'           \item \strong{\code{"validate"}}: Same as \code{"estimate"}, but it also runs a set of tests to validate the function inputs.
+#'           \item \strong{\code{"zero_LL"}}: List with probabilities for each class in an equal shares setting.
 #'         }
 #' @export
 #' @importFrom utils capture.output
@@ -51,6 +53,10 @@ apollo_classAlloc <- function(classAlloc_settings){
     assign("apollo_modelList", apollo_modelList, envir=parent.frame())
   }
   
+  #### replace utilities by V if used, and "alternatives" by "classes".
+  if(!is.null(classAlloc_settings[["utilities"]])) names(classAlloc_settings)[which(names(classAlloc_settings)=="utilities")]="V"
+  if(!is.null(classAlloc_settings[["alternatives"]])) names(classAlloc_settings)[which(names(classAlloc_settings)=="alternatives")]="classes"
+  
   # ############################### #
   #### Load or do pre-processing ####
   # ############################### #
@@ -129,7 +135,7 @@ apollo_classAlloc <- function(classAlloc_settings){
   
   if(functionality=="validate"){
     
-    if(!apollo_inputs$apollo_control$noValidation)apollo_validate(classAlloc_settings, modelType, 
+    if(!apollo_inputs$apollo_control$noValidation) apollo_validate(classAlloc_settings, modelType, 
                                                                   functionality, apollo_inputs)
     
     # No diagnose for the class allocation component
