@@ -1,37 +1,45 @@
 #' Calculates Fractional Multinomial Logit probabilities
 #'
-#' Calculates probabilities of a Fractional Multinomial Logit model.
+#' Calculates the probabilities of a Fractional Multinomial Logit model and can also perform other operations based on the value of the \code{functionality} argument.
 #'
 #' @param fmnl_settings List of inputs of the FMNL model. It should contain the following.
 #'                     \itemize{
-#'                       \item \strong{\code{alternatives}}: Named numeric vector. Names of alternatives and their corresponding value in \code{choiceVar}.
-#'                       \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1.
+#'                       \item \strong{\code{alternatives}}: Character vector. Names of alternatives, elements must match the names in list 'utilities'.
+#'                       \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1. These can be scalars or vectors (of length equal to rows in the database). A user can also specify \code{avail=1} to indicate universal availability, or omit the setting completely.
 #'                       \item \strong{\code{choiceShares}}: Named list of numeric vectors. Share allocated to each alternative. One element per alternative, as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
-#'                       \item \strong{\code{V}}: Named list of deterministic utilities . Utilities of the alternatives. Names of elements must match those in \code{alternatives.}
-#'                       \item \strong{\code{rows}}: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
 #'                       \item \strong{\code{componentName}}: Character. Name given to model component.
+#'                       \item \strong{\code{rows}}: Boolean vector. Consideration of which rows to include. Length equal to the number of observations (nObs), with entries equal to TRUE for rows to include, and FALSE for rows to exclude. Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                       \item \strong{\code{utilities}}: Named list of deterministic utilities . Utilities of the alternatives. Names of elements must match those in \code{alternatives.}
 #'                     }
-#' @param functionality Character. Can take different values depending on desired output.
+#' @param functionality Character. Setting instructing Apollo what processing to apply to the likelihood function. This is in general controlled by the functions that call \code{apollo_probabilities}, though the user can also call \code{apollo_probabilities} manually with a given functionality for testing/debugging. Possible values are:
 #'                      \itemize{
-#'                        \item \code{"estimate"}: Used for model estimation.
-#'                        \item \code{"prediction"}: Used for model predictions.
-#'                        \item \code{"validate"}: Used for validating input.
-#'                        \item \code{"zero_LL"}: Used for calculating null likelihood.
-#'                        \item \code{"shares_LL"}: Used for calculating likelihood with constants only.
-#'                        \item \code{"conditionals"}: Used for calculating conditionals.
-#'                        \item \code{"output"}: Used for preparing output after model estimation.
-#'                        \item \code{"raw"}: Used for debugging.
+#'                        \item \strong{\code{"components"}}: For further processing/debugging, produces likelihood for each model component (if multiple components are present), at the level of individual draws and observations.
+#'                        \item \strong{\code{"conditionals"}}: For conditionals, produces likelihood of the full model, at the level of individual inter-individual draws.
+#'                        \item \strong{\code{"estimate"}}: For model estimation, produces likelihood of the full model, at the level of individual decision-makers, after averaging across draws.
+#'                        \item \strong{\code{"gradient"}}: For model estimation, produces analytical gradients of the likelihood, where possible.
+#'                        \item \strong{\code{"output"}}: Prepares output for post-estimation reporting.
+#'                        \item \strong{\code{"prediction"}}: For model prediction, produces probabilities for individual alternatives and individual model components (if multiple components are present) at the level of an observation, after averaging across draws.
+#'                        \item \strong{\code{"preprocess"}}: Prepares likelihood functions for use in estimation.
+#'                        \item \strong{\code{"raw"}}: For debugging, produces probabilities of all alternatives and individual model components at the level of an observation, at the level of individual draws.
+#'                        \item \strong{\code{"report"}}: Prepares output summarising model and choiceset structure.
+#'                        \item \strong{\code{"shares_LL"}}: Produces overall model likelihood with constants only.
+#'                        \item \strong{\code{"validate"}}: Validates model specification, produces likelihood of the full model, at the level of individual decision-makers, after averaging across draws.
+#'                        \item \strong{\code{"zero_LL"}}: Produces overall model likelihood with all parameters at zero.
 #'                      }
 #' @return The returned object depends on the value of argument \code{functionality} as follows.
 #'         \itemize{
+#'           \item \strong{\code{"components"}}: Same as \code{"estimate"}
+#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}
 #'           \item \strong{\code{"estimate"}}: vector/matrix/array. Returns the probabilities for the chosen alternative for each observation.
+#'           \item \strong{\code{"gradient"}}: List containing the likelihood and gradient of the model component.
+#'           \item \strong{\code{"output"}}: Same as \code{"estimate"} but also writes summary of input data to internal Apollo log.
 #'           \item \strong{\code{"prediction"}}: List of vectors/matrices/arrays. Returns a list with the probabilities for all alternatives, with an extra element for the probability of the chosen alternative.
+#'           \item \strong{\code{"preprocess"}}: Returns a list with pre-processed inputs, based on \code{mfnl_settings}.
+#'           \item \strong{\code{"raw"}}: Same as \code{"prediction"}
+#'           \item \strong{\code{"report"}}: Overview of dependent variable
+#'           \item \strong{\code{"shares_LL"}}: vector/matrix/array. Returns the probability of the chosen alternative when only constants are estimated.
 #'           \item \strong{\code{"validate"}}: Same as \code{"estimate"}, but it also runs a set of tests to validate the function inputs.
 #'           \item \strong{\code{"zero_LL"}}: vector/matrix/array. Returns the probability of the chosen alternative when all parameters are zero.
-#'           \item \strong{\code{"shares_LL"}}: vector/matrix/array. Returns the probability of the chosen alternative when only constants are estimated.
-#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}
-#'           \item \strong{\code{"output"}}: Same as \code{"estimate"} but also writes summary of input data to internal Apollo log.
-#'           \item \strong{\code{"raw"}}: Same as \code{"prediction"}
 #'         }
 #' @export
 #' @importFrom utils capture.output
@@ -58,6 +66,12 @@ apollo_fmnl <- function(fmnl_settings, functionality){
     assign("apollo_modelList", apollo_modelList, envir=parent.frame())
   }
   
+  ### replace utilities by V if used
+  if(!is.null(fmnl_settings[["utilities"]])) names(fmnl_settings)[which(names(fmnl_settings)=="utilities")]="V"
+  
+  ### turn character vector of alternatives into named vector (to make compatible with MNL code)
+  if(is.character(fmnl_settings[["alternatives"]])) fmnl_settings[["alternatives"]]=setNames(1:length(fmnl_settings[["alternatives"]]), fmnl_settings[["alternatives"]])
+  
   # ############################### #
   #### Load or do pre-processing ####
   # ############################### #
@@ -78,7 +92,7 @@ apollo_fmnl <- function(fmnl_settings, functionality){
     # Determine which fmnl likelihood to use (R or C++)
     if(apollo_inputs$apollo_control$cpp & !apollo_inputs$silent) apollo_print("No C++ optimisation for fmnl available")
     # Using R likelihood
-    fmnl_settings$probs_fmnl=function(fmnl_settings, all=FALSE, restoreRows=TRUE){
+    fmnl_settings$probs_fmnl=function(fmnl_settings, all=FALSE){#}, restoreRows=TRUE){
       # Fix choiceVar if "raw" and choiceVar==NA
       fmnl_settings$choiceNA = FALSE
       if(all(is.na(unlist(fmnl_settings$choiceShares)))){
@@ -131,10 +145,10 @@ apollo_fmnl <- function(fmnl_settings, functionality){
         ### SH TO CHANGE
         P <- numerator/denom
       }
-      # insert excluded rows with value 1 if only teh chosen is requested, and 0 if all
-      if(any(!fmnl_settings$rows) & restoreRows){
-        if(is.list(P)) P <- lapply(P, apollo_insertRows, r=fmnl_settings$r, val=0) else P <- apollo_insertRows(P, fmnl_settings$rows, 1)
-      }
+      ## insert excluded rows with value 1 if only teh chosen is requested, and 0 if all
+      #if(any(!fmnl_settings$rows) & restoreRows){
+      #  if(is.list(P)) P <- lapply(P, apollo_insertRows, r=fmnl_settings$r, val=0) else P <- apollo_insertRows(P, fmnl_settings$rows, 1)
+      #}
       return(P)
     }
     
@@ -180,7 +194,7 @@ apollo_fmnl <- function(fmnl_settings, functionality){
   
   if(functionality=="validate"){
     # Check that alternatives are named in altcodes and V
-    if(is.null(fmnl_settings$altnames) || is.null(fmnl_settings$altcodes) || is.null(names(fmnl_settings$V))) stop("Alternatives for model component \"",fmnl_settings$componentName,"\" must be named, both in 'alternatives' and 'V'.")
+    if(is.null(fmnl_settings$altnames) || is.null(fmnl_settings$altcodes) || is.null(names(fmnl_settings$V))) stop("Alternatives for model component \"",fmnl_settings$componentName,"\" must be named, both in 'alternatives' and 'utilities'.")
     
     if(!apollo_inputs$apollo_control$noValidation) apollo_validate(fmnl_settings, modelType, 
                                                                    functionality, apollo_inputs)
@@ -188,6 +202,7 @@ apollo_fmnl <- function(fmnl_settings, functionality){
     if(!apollo_inputs$apollo_control$noDiagnostics) apollo_diagnostics(fmnl_settings, modelType, apollo_inputs)
     
     testL = fmnl_settings$probs_fmnl(fmnl_settings, all=FALSE)
+    if(any(!fmnl_settings$rows)) testL <- apollo_insertRows(testL, fmnl_settings$rows, 1)
     if(all(testL==0)) stop("All observations have zero probability at starting value for model component \"",fmnl_settings$componentName,"\"")
     if(any(testL==0) && !apollo_inputs$silent && apollo_inputs$apollo_control$debug) apollo_print(paste0("Some observations have zero probability at starting value for model component \"",fmnl_settings$componentName,"\"", sep=""))
     return(invisible(testL))
@@ -233,11 +248,15 @@ apollo_fmnl <- function(fmnl_settings, functionality){
   # ################################################################# #
   
   if(functionality %in% c("estimate","conditionals", "components", "output")){
-    return(fmnl_settings$probs_fmnl(fmnl_settings, all=FALSE))
+    P <- fmnl_settings$probs_fmnl(fmnl_settings, all=FALSE)
+    if(any(!fmnl_settings$rows)) P <- apollo_insertRows(P, fmnl_settings$rows, 1)
+    return(P)
   }
   
   if(functionality %in% c("prediction","raw")){
-    return(fmnl_settings$probs_fmnl(fmnl_settings, all=TRUE))
+    P <- fmnl_settings$probs_fmnl(fmnl_settings, all=TRUE)
+    if(any(!fmnl_settings$rows)) P <- lapply(P, apollo_insertRows, r=fmnl_settings$rows, val=NA)
+    return(P)
   }
   
   # ############################## #
@@ -252,7 +271,7 @@ apollo_fmnl <- function(fmnl_settings, functionality){
    if(is.null(apollo_inputs$database)) stop("apollo_fmnl could not fetch apollo_inputs$database for gradient estimation.")
    
    # Calculate probabilities and derivatives of utilities for all alternatives
-   P    <- fmnl_settings$probs_fmnl(fmnl_settings, all=TRUE, restoreRows=FALSE)
+   P    <- fmnl_settings$probs_fmnl(fmnl_settings, all=TRUE)
    Pcho <- P[["chosen"]]
    P    <- P[-which(names(P)=="chosen")]
    e <- list2env(c(as.list(apollo_beta), apollo_inputs$database, list(apollo_inputs=apollo_inputs)), hash=TRUE)
