@@ -1,45 +1,52 @@
-#' Calculates MDCNEV likelihoods with an outside good.
+#' Calculates MDCNEV likelihoods
 #' 
-#' Calculates the likelihood of a Multiple Discrete Continuous Nested Extreme Value (MDCNEV) model with an outside good.
+#' Calculates the likelihoods of a Multiple Discrete Continuous Nested Extreme Value (MDCNEV) model with an outside good and can also perform other operations based on the value of the \code{functionality} argument.
 #' 
-#' @param mdcnev_settings List of settings for the MDCEV model. It must include the following.
+#' @param mdcnev_settings List. Contains settings for this function. User input is required for all settings except those with a default or marked as optional. 
 #'                       \itemize{
-#'                         \item \strong{\code{V}}: Named list. Utilities of the alternatives. Names of elements must match those in argument 'alternatives'.
-#'                         \item \strong{\code{alternatives}}: Character vector. Names of alternatives, elements must match the names in list 'V'.
 #'                         \item \strong{\code{alpha}}: Named list. Alpha parameters for each alternative, including for the outside good. As many elements as alternatives.
+#'                         \item \strong{\code{avail}}: Named list of numeric vectors or scalars. Availabilities of alternatives, one element per alternative. Names of elements must match those in \code{alternatives}. Values can be 0 or 1. These can be scalars or vectors (of length equal to rows in the database). A user can also specify \code{avail=1} to indicate universal availability, or omit the setting completely.
+#'                         \item \strong{\code{alternatives}}: Character vector. Names of alternatives, elements must match the names in list 'utilities'.
+#'                         \item \strong{\code{budget}}: Numeric vector. Budget for each observation.
+#'                         \item \strong{\code{componentName}}: Character. Name given to model component.
+#'                         \item \strong{\code{continuousChoice}}: Named list of numeric vectors. Amount of consumption of each alternative. One element per alternative, as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
+#'                         \item \strong{\code{cost}}: Named list of numeric vectors. Price of each alternative. One element per alternative, each one as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
 #'                         \item \strong{\code{gamma}}: Named list. Gamma parameters for each alternative, including for the outside good. As many elements as alternatives.
 #'                         \item \strong{\code{mdcnevNests}}: Named list. Lambda parameters for each nest. Elements must be named with the nest name. The lambda at the root is fixed to 1, and therefore must be no be defined. The value of the estimated mdcnevNests parameters should be between 0 and 1 to ensure consistency with random utility maximization.
 #'                         \item \strong{\code{mdcnevStructure}}: Numeric matrix. One row per nest and one column per alternative. Each element of the matrix is 1 if an alternative belongs to the corresponding nest.
-#'                         \item \strong{\code{cost}}: Named list of numeric vectors. Price of each alternative. One element per alternative, each one as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
-#'                         \item \strong{\code{avail}}: Named list. Availabilities of alternatives, one element per alternative. Names of elements must match those in argument 'alternatives'. Value for each element can be 1 (scalar if always available) or a vector with values 0 or 1 for each observation. If all alternatives are always available, then user can just omit this argument.
-#'                         \item \strong{\code{continuousChoice}}: Named list of numeric vectors. Amount of consumption of each alternative. One element per alternative, as long as the number of observations or a scalar. Names must match those in \code{alternatives}.
-#'                         \item \strong{\code{budget}}: Numeric vector. Budget for each observation.
-#'                         \item \strong{\code{minConsumption}}: Named list of scalars or numeric vectors. Minimum consumption of the alternatives, if consumed. As many elements as alternatives. Names must match those in \code{alternatives}.
 #'                         \item \strong{\code{outside}}: Character. Alternative name for the outside good. Default is "outside"
-#'                         \item \strong{\code{rows}}: Boolean vector. Consideration of rows in the likelihood calculation, FALSE to exclude. Length equal to the number of observations (nObs). Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
-#'                         \item \strong{\code{componentName}}: Character. Name given to model component.
+#'                         \item \strong{\code{rows}}: Boolean vector. Consideration of which rows to include. Length equal to the number of observations (nObs), with entries equal to TRUE for rows to include, and FALSE for rows to exclude. Default is \code{"all"}, equivalent to \code{rep(TRUE, nObs)}.
+#'                         \item \strong{\code{utilities}}: Named list. Utilities of the alternatives. Names of elements must match those in argument 'alternatives'.
 #'                       }
-#' @param functionality Character. Can take different values depending on desired output.
+#' @param functionality Character. Setting instructing Apollo what processing to apply to the likelihood function. This is in general controlled by the functions that call \code{apollo_probabilities}, though the user can also call \code{apollo_probabilities} manually with a given functionality for testing/debugging. Possible values are:
 #'                      \itemize{
-#'                        \item "estimate" Used for model estimation.
-#'                        \item "prediction" Used for model predictions.
-#'                        \item "validate" Used for validating input.
-#'                        \item "zero_LL" Used for calculating null likelihood.
-#'                        \item "shares_LL" Used for calculating likelihood with constants only.
-#'                        \item "conditionals" Used for calculating conditionals.
-#'                        \item "output" Used for preparing output after model estimation.
-#'                        \item "raw" Used for debugging.
+#'                        \item \strong{\code{"components"}}: For further processing/debugging, produces likelihood for each model component (if multiple components are present), at the level of individual draws and observations.
+#'                        \item \strong{\code{"conditionals"}}: For conditionals, produces likelihood of the full model, at the level of individual inter-individual draws.
+#'                        \item \strong{\code{"estimate"}}: For model estimation, produces likelihood of the full model, at the level of individual decision-makers, after averaging across draws.
+#'                        \item \strong{\code{"gradient"}}: For model estimation, produces analytical gradients of the likelihood, where possible.
+#'                        \item \strong{\code{"output"}}: Prepares output for post-estimation reporting.
+#'                        \item \strong{\code{"prediction"}}: For model prediction, produces probabilities for individual alternatives and individual model components (if multiple components are present) at the level of an observation, after averaging across draws.
+#'                        \item \strong{\code{"preprocess"}}: Prepares likelihood functions for use in estimation.
+#'                        \item \strong{\code{"raw"}}: For debugging, produces probabilities of all alternatives and individual model components at the level of an observation, at the level of individual draws.
+#'                        \item \strong{\code{"report"}}: Prepares output summarising model and choiceset structure.
+#'                        \item \strong{\code{"shares_LL"}}: Produces overall model likelihood with constants only.
+#'                        \item \strong{\code{"validate"}}: Validates model specification, produces likelihood of the full model, at the level of individual decision-makers, after averaging across draws.
+#'                        \item \strong{\code{"zero_LL"}}: Produces overall model likelihood with all parameters at zero.
 #'                      }
 #' @return The returned object depends on the value of argument \code{functionality} as follows.
 #'         \itemize{
+#'           \item \strong{\code{"components"}}: Same as \code{"estimate"}
+#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}
 #'           \item \strong{\code{"estimate"}}: vector/matrix/array. Returns the probabilities for the observed consumption for each observation.
+#'           \item \strong{\code{"gradient"}}: Not implemented
+#'           \item \strong{\code{"output"}}: Same as \code{"estimate"} but also writes summary of input data to internal Apollo log.
 #'           \item \strong{\code{"prediction"}}: A matrix with one row per observation, and columns indicating means and s.d. of continuous and discrete predicted consumptions.
+#'           \item \strong{\code{"preprocess"}}: Returns a list with pre-processed inputs, based on \code{mdcnev_settings}.
+#'           \item \strong{\code{"raw"}}: Same as \code{"estimate"}
+#'           \item \strong{\code{"report"}}: Dependent variable overview.
+#'           \item \strong{\code{"shares_LL"}}: Not implemented. Returns a vector of NA with as many elements as observations.
 #'           \item \strong{\code{"validate"}}: Same as \code{"estimate"}, but it also runs a set of tests to validate the function inputs.
 #'           \item \strong{\code{"zero_LL"}}: Not implemented. Returns a vector of NA with as many elements as observations.
-#'           \item \strong{\code{"shares_LL"}}: Not implemented. Returns a vector of NA with as many elements as observations.
-#'           \item \strong{\code{"conditionals"}}: Same as \code{"estimate"}
-#'           \item \strong{\code{"output"}}: Same as \code{"estimate"} but also writes summary of input data to internal Apollo log.
-#'           \item \strong{\code{"raw"}}: Same as \code{"estimate"}
 #'         }
 #' @importFrom mnormt rmnorm
 #' @importFrom stats setNames
@@ -315,10 +322,10 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
     return(P)
   }
   
-  # ################################################## #
-  #### functionality="estimate/conditionals/output" ####
-  # ################################################## #
-  if(functionality %in% c("estimate", "conditionals", "output")){
+  # ############################################################# #
+  #### functionality="estimate/conditionals/output/components" ####
+  # ############################################################# #
+  if(functionality %in% c("estimate", "conditionals", "output", "components")){
     L <- mdcnev_settings$probs_MDCNEV(mdcnev_settings)
     if(any(!mdcnev_settings$rows)) L <- apollo_insertRows(L, mdcnev_settings$rows, 1)
     return(L)
@@ -387,9 +394,10 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
     step1 <- ceiling(s$nRep/10)
     step2 <- ceiling(s$nRep/2)
     
-    # Expand avail and cost
+    # Expand avail, cost and budget
     avail <- sapply(s$avail, function(a) if(length(a)==1) rep(a, s$nObs) else a)
     cost  <- sapply(s$cost , function(x) if(length(x)==1) rep(x, s$nObs) else x)
+    if(length(s$budget)==1) budget <- rep(s$budget, s$nObs) else budget <- s$budget
     
     ### Simulate prediction
     if(!is.null(apollo_inputs$silent)) silent <- apollo_inputs$silent else silent <- FALSE
@@ -407,7 +415,7 @@ apollo_mdcnev <- function(mdcnev_settings,functionality){
           # Calculate prediction
           for(i in 1:s$nObs){
             p  <- cost[i,2:s$nAlt]
-            b  <- s$budget[i]
+            b  <- budget[i]
             g  <- gamma[i,2:s$nAlt]
             a0 <- alpha[i,1]
             ak <- alpha[i,2:s$nAlt]
